-----------------------------------------------------------------
-- (c) 2007 VUT-FEL, K336
-- http://cs.felk.cvut.cz
-----------------------------------------------------------------
-- Filename : proctb.vhd
-- Revision : V1.1
-- Author :   Josef Nouzk
-- Created :  22. 04. 2007
--
-- Modifications
-- 2.0	04. 05. 2007 by Zdenk Sojka
-- 1.1	22. 04. 2007 by Zdenk Sojka
-- 1.0	22. 04. 2007 by Josef Nouzk
-----------------------------------------------------------------
-- testbench pro proc - zkou zkladn funkci nsoben
--
-- 2.0	odstrann signlu 'load', pidn signlu 'valid'
-- 1.1	opraveno asovn
-----------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_unsigned.all;
USE ieee.numeric_std.ALL;

ENTITY proctb_vhd IS
END proctb_vhd;

ARCHITECTURE behavior OF proctb_vhd IS 

	-- Component Declaration for the Unit Under Test (UUT)
	COMPONENT proc
	PORT(
		btns : IN std_logic_vector(1 downto 0);
		swtch : IN std_logic_vector(7 downto 0);
		clk : IN std_logic;
		rst : IN std_logic;          
		numA : OUT std_logic_vector(7 downto 0);
		numB : OUT std_logic_vector(7 downto 0);
		valid : OUT std_logic;
		vys : OUT std_logic_vector(15 downto 0)
		);
	END COMPONENT;

	--Inputs
	SIGNAL clk :  std_logic := '0';
	SIGNAL rst :  std_logic := '0';
	SIGNAL btns :  std_logic_vector(1 downto 0) := (others=>'0');
	SIGNAL swtch :  std_logic_vector(7 downto 0) := (others=>'0');

	--Outputs
	SIGNAL numA :  std_logic_vector(7 downto 0);
	SIGNAL numB :  std_logic_vector(7 downto 0);
	SIGNAL valid :  std_logic;
	SIGNAL vys :  std_logic_vector(15 downto 0);

BEGIN

	-- Instantiate the Unit Under Test (UUT)
	uut: proc PORT MAP(
		btns => btns,
		swtch => swtch,
		numA => numA,
		numB => numB,
		valid => valid,
		vys => vys,
		clk => clk,
		rst => rst
	);

 	hodiny : process
	
	begin
		clk <= '0';
		wait for 10 ns;
		clk <= '1';
		wait for 10 ns;
	end process;

	tb : PROCESS
	
	variable err_cnt: integer :=0;
	
	BEGIN

		-- Wait 100 ns for global reset to finish
		rst<='1';
		wait for 100 ns;
		rst<='0';
		-- Place stimulus here
		-----case1-------
		wait for 100 ns;
		swtch<="00000001";
		btns(0)<='1';
		wait for 100 ns;
		btns(0)<='0';
		wait for 400 ns;
		swtch<="00010001";
		btns(1)<='1';
		wait for 100 ns;
		btns(1)<='0';
		wait for 400 ns;
		assert (vys="00010001") report "Error! chyba 1 vypoctu" severity error;
		if (vys/="00010001") then
			err_cnt:=err_cnt+1;
		end if;

		---case2-------
		wait for 15ns;
		swtch<="11111111";
		btns(0)<='1';
		wait for 100 ns;
		btns(0)<='0';
		wait for 400 ns;
		swtch<="00000001";
		btns(1)<='1';
		wait for 100 ns;
		btns(1)<='0';
		wait for 400 ns;
		assert (vys="11111111") report "Error! chyba 1 vypoctu" severity error;
		if (vys/="11111111") then
			err_cnt:=err_cnt+1;
		end if;

		------case3---------
		wait for 15ns;
		
		if (err_cnt=0) then
	    assert false
	    report "Testbench of Proc completed successfully!"
	    severity note;
		else
	    assert true
	    report "Something wrong, try again"
	    severity error;
		end if;

		wait; -- will wait forever
	END PROCESS;

END;


