-----------------------------------------------------------------
-- (c) 2007 VUT-FEL, K336
-- http://cs.felk.cvut.cz
-----------------------------------------------------------------
-- Filename : registrB_bench.vhd
-- Revision : V1.0
-- Author :   Jan Skalicky
-- Created :  22. 04. 2007
--
-- Modifications
-- 
-----------------------------------------------------------------
-- test bench for this entity
--
-- 1.0
-----------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_unsigned.all;
USE ieee.numeric_std.ALL;

ENTITY registrB_bench_vhd IS
END registrB_bench_vhd;

ARCHITECTURE behavior OF registrB_bench_vhd IS 

	-- Component Declaration for the Unit Under Test (UUT)
	COMPONENT registrB
	PORT(
		sw : IN std_logic_vector(15 downto 0);
		set : IN std_logic;
		shl : IN std_logic;
		clk : IN std_logic;
		rst : IN std_logic;          
		val : OUT std_logic_vector(15 downto 0)
		);
	END COMPONENT;

	--Inputs
	SIGNAL set :  std_logic := '0';
	SIGNAL shl :  std_logic := '0';
	SIGNAL clk :  std_logic := '0';
	SIGNAL rst :  std_logic := '0';
	SIGNAL sw :  std_logic_vector(15 downto 0) := (others=>'0');

	--Outputs
	SIGNAL val :  std_logic_vector(15 downto 0);

BEGIN

	-- Instantiate the Unit Under Test (UUT)
	uut: registrB PORT MAP(
		sw => sw,
		set => set,
		shl => shl,
		val => val,
		clk => clk,
		rst => rst
	);

	tb : process
	 begin

		rst <= '1';
		wait for 10ns;
		rst <= '0';
		
		-- testing certain data with various level widths
		wait until clk'event and clk = '0';
		sw <= "1111100001110010";
		set <= '1';
		shl <= '0';
		wait until clk'event and clk = '0';
		assert( val = "1111100001110010" );
		wait until clk'event and clk = '0';
		set <= '0';
		shl <= '1';
		wait until clk'event and clk = '0';
		assert( val = "1111000011100100" );
		wait until clk'event and clk = '0';
		assert( val = "1110000111001000" );
		wait until clk'event and clk = '0';
		assert( val = "1100001110010000" );
		wait until clk'event and clk = '0';
		assert( val = "1000011100100000" );
		wait until clk'event and clk = '0';
		assert( val = "0000111001000000" );
		wait until clk'event and clk = '0';
		assert( val = "0001110010000000" );
		wait until clk'event and clk = '0';
		assert( val = "0011100100000000" );
		wait until clk'event and clk = '0';
		assert( val = "0111001000000000" );
		wait until clk'event and clk = '0';
		assert( val = "1110010000000000" );
		wait until clk'event and clk = '0';
		assert( val = "1100100000000000" );
		wait until clk'event and clk = '0';
		assert( val = "1001000000000000" );
		wait until clk'event and clk = '0';
		assert( val = "0010000000000000" );
		wait until clk'event and clk = '0';
		assert( val = "0100000000000000" );
		wait until clk'event and clk = '0';
		assert( val = "1000000000000000" );
		wait until clk'event and clk = '0';
		assert( val = "0000000000000000" );
		wait until clk'event and clk = '0';
		-- now, registr is empty (filled with 0)
		shl <= '0';
		assert( val = "0000000000000000" );

		report "Testbench finished, check for failed assertions" severity note;
		wait;
	 end process;
	
	clock : process
	 begin
		clk <= '0';
		wait for 100ns;
		clk <= '1';
		wait for 100ns;
	 end process;
	
END;
