-----------------------------------------------------------------
-- (c) 2007 VUT-FEL, K336
-- http://cs.felk.cvut.cz
-----------------------------------------------------------------
-- Filename : registrBtb.vhd
-- Revision : V1.1
-- Author :   Josef Nouzk
-- Created :  22. 04. 2007
--
-- Modifications
-- 1.1	22. 04. 2007 by Zdenk Sojka
-- 1.0	22. 04. 2007 by Josef Nouzk
-----------------------------------------------------------------
-- testbench pro registr B - zkou natn a posun
--
-- 1.1	upraveno asovn
-----------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_unsigned.all;
USE ieee.numeric_std.ALL;

ENTITY registrBtb_vhd IS
END registrBtb_vhd;

ARCHITECTURE behavior OF registrBtb_vhd IS 

	-- Component Declaration for the Unit Under Test (UUT)
	COMPONENT registrB
	PORT(
		sw : IN std_logic_vector(15 downto 0);
		set : IN std_logic;
		shl : IN std_logic;
		clk : IN std_logic;
		rst : IN std_logic;          
		val : OUT std_logic_vector(15 downto 0)
		);
	END COMPONENT;

	--Inputs
	SIGNAL set :  std_logic := '0';
	SIGNAL shl :  std_logic := '0';
	SIGNAL clk :  std_logic := '0';
	SIGNAL rst :  std_logic := '0';
	SIGNAL sw :  std_logic_vector(15 downto 0) := (others=>'0');

	--Outputs
	SIGNAL val :  std_logic_vector(15 downto 0);

BEGIN

	-- Instantiate the Unit Under Test (UUT)
	uut: registrB PORT MAP(
		sw => sw,
		set => set,
		shl => shl,
		val => val,
		clk => clk,
		rst => rst
	);
	
	process
		begin
			clk<='0';
			wait for 10ns;
			clk<='1';
			wait for 10ns;
	end process;
	
	tb : PROCESS
	
	variable err_cnt: integer :=0;
	
	BEGIN

		-- Wait 100 ns for global reset to finish
		wait for 100 ns;
		rst <= '1';
		wait for 40ns;
		rst <= '0';
		wait for 40ns;
		-- Place stimulus here
		
		----case1-------set
		sw <="1010101010101010";
		set <='1';
		wait for 80ns;
		set <='0';
		wait for 40ns;
		assert (val="1010101010101010") report "load Error!" severity error;
		if (val/="1010101010101010") then
			err_cnt:=err_cnt+1;
		end if;
		------case2-------shl
		wait for 40ns;
		shl <='1';
		wait for 20ns;
		shl <='0';
		wait for 100ns;
		assert (val="0101010101010100") report "shl Error!" severity error;
		if (val/="0101010101010100") then
			err_cnt:=err_cnt+1;
		end if;
		
		
		
		----case3-------reset
		wait for 40ns;
		rst <= '1';
		wait for 40ns;
		rst <= '0';
		wait for 40ns;
		assert (val="0000000000000000") report "reset Error!" severity error;
		if (val/="0000000000000000") then
			err_cnt:=err_cnt+1;
		end if;
		
		-- summary of testbench
		if (err_cnt=0) then
	    assert false
	    report "Testbench of Adder completed successfully!"
	    severity note;
		else
	    assert true
	    report "Something wrong, try again"
	    severity error;
		end if;
		
		wait; -- will wait forever
	END PROCESS;

END;
